// JmolJSmol.js -- Jmol pure JavaScript version
// author: Bob Hanson, hansonr@stolaf.edu	4/16/2012
// author: Takanori Nakane biochem_fan 6/12/2012

// This library requires
//
//	jQuery.min.js
//	JmolCore.js
//  JmolApplet.js
//  JmolApi.js
//  Three49custom.js
//  GLmol.js
//  JmolGLmol.js
//  j2sjmol.js    (Clazz and associated classes)
// prior to JmolJSmol.js

(function (Jmol) {

	Jmol._getCanvas = function(id, Info, checkOnly, checkWebGL, checkHTML5) {
		// overrides the function in JmolCore.js
		var canvas = null;
		try{
		  LoadClazz();
		} catch (e) {
			return null;
		}
		if (checkWebGL && Jmol.featureDetection.supportsWebGL()) {
			Jmol._Canvas3D.prototype = Jmol._jsSetPrototype(new Jmol._Applet(id,Info, "", true));
			canvas = new Jmol._Canvas3D(id, Info, null, checkOnly);
		}
		if (checkHTML5 && canvas == null) {
			Jmol._Canvas2D.prototype = Jmol._jsSetPrototype(new Jmol._Applet(id,Info, "", true));
			canvas = new Jmol._Canvas2D(id, Info, null, checkOnly);
		}
		return canvas;
	};

	Jmol._Canvas3D = function(id, Info, caption, checkOnly){
		this._syncId = ("" + Math.random()).substring(3);
		this._id = id;
		this._is2D = false;
		this._jmolType = "Jmol._Canvas3D (JSmol)";
		this._platform = "org.jmol.awtjs.Platform";
		if (checkOnly)
			return this;
		window[id] = this;
		this._create(id, Info, caption);
		this._setupJS(id, Info);
		this._showInfo(true); // until we have graphics
		return this;
	};

	Jmol._Canvas2D = function(id, Info, caption, checkOnly){
		this._syncId = ("" + Math.random()).substring(3);
		this._id = id;
		this._is2D = true;
		this._jmolType = "Jmol._Canvas2D (JSmol)";
		this._platform = "org.jmol.awtjs2d.Platform";
		if (checkOnly)
			return this;
		window[id] = this;
		this._create2D(id, Info, caption);
		this._setupJS(id, Info);
		this._showInfo(true); // until we have graphics
		return this;
	};

	Jmol._jsGetMouseModifiers = function(ev) {
		var modifiers = 0;
		switch (ev.button) {
		case 0:
		  modifiers = org.jmol.api.Event.MOUSE_LEFT;
		  break;
		case 1:
		  modifiers = org.jmol.api.Event.MOUSE_MIDDLE;
		  break;
		case 2:
		  modifiers = org.jmol.api.Event.MOUSE_RIGHT;
		  break;
		}
		if (ev.shiftKey)
		  modifiers += org.jmol.api.Event.SHIFT_MASK;
		if (ev.altKey)
		  modifiers += org.jmol.api.Event.ALT_MASK;
		if (ev.ctrlKey)
		  modifiers += org.jmol.api.Event.CTRL_MASK;
		//document.title = ev.button + " " + ev.type + " " + ev.which
		return modifiers;
	}

	Jmol._jsGetXY = function(ev) {
		ev.preventDefault();
		var x = ev.pageX, y = ev.pageY;
		if (ev.originalEvent.targetTouches && ev.originalEvent.targetTouches[0]) {
			x = ev.originalEvent.targetTouches[0].pageX;
			y = ev.originalEvent.targetTouches[0].pageY;
		}
		return (x == undefined ? null : [x, y, Jmol._jsGetMouseModifiers(ev)]);
	}

	Jmol._jsSetMouse = function(canvas) {
		var d = $(canvas);
		d.bind('mouseup touchend', function(ev) {
		  canvas.isDragging = false;
			var xym = Jmol._jsGetXY(ev);
			if(!xym) return;
			canvas.applet._applet.viewer.handleOldJvm10Event(org.jmol.api.Event.MOUSE_UP,xym[0],xym[1],xym[2],System.currentTimeMillis());
		});
		d.bind('mousedown touchstart', function(ev) {
			var xym = Jmol._jsGetXY(ev);
			if(!xym) return;
		  canvas.isDragging = true;
			canvas.applet._applet.viewer.handleOldJvm10Event(org.jmol.api.Event.MOUSE_DOWN,xym[0],xym[1],xym[2],System.currentTimeMillis());
		});

		d.bind('DOMMouseScroll mousewheel', function(ev) { // Zoom
			ev.preventDefault();
			// Webkit or Firefox
		  canvas.isDragging = false;
			var scroll = (ev.originalEvent.detail ? ev.originalEvent.detail : ev.originalEvent.wheelDelta);
			var modifiers = Jmol._jsGetMouseModifiers(ev);
			canvas.applet._applet.viewer.handleOldJvm10Event(-1,scroll < 0 ? -1 : 1,0,modifiers,System.currentTimeMillis());
		});
		d.bind("contextmenu", function(ev) {ev.preventDefault();});
		$('body').bind('mouseup touchend', function(ev) {
			canvas.isDragging = false;
		});

		d.bind('mouseenter', function(ev) {
		if (ev.buttons === 0 || ev.which === 0) {
		  canvas.isDragging = false;
			var xym = Jmol._jsGetXY(ev);
			canvas.applet._applet.viewer.handleOldJvm10Event(org.jmol.api.Event.MOUSE_UP,xym[0],xym[1],xym[2],System.currentTimeMillis());
		}
		});

		d.bind('mousemove touchmove', function(ev) { // touchmove
			var xym = Jmol._jsGetXY(ev);
			if(!xym) return;
			canvas.applet._applet.viewer.handleOldJvm10Event(
				(canvas.isDragging ? org.jmol.api.Event.MOUSE_DRAG : org.jmol.api.Event.MOUSE_MOVE),
				xym[0], xym[1], (canvas.isDragging ? xym[2] : 0), System.currentTimeMillis()
			);
		});
	}

	Jmol._jsSetPrototype = function(proto) {
		proto._create = function(id, Info, caption) {
			Jmol._setObject(this, id, Info);
			var t = Jmol._getWrapper(this, true);
			//console.log(this);
			this._GLmol = new GLmol;
			this._GLmol.applet = this;

			if (Jmol._document) {
				Jmol._documentWrite(t);
				this._GLmol.create(id + "_appletdiv", true);
				t = "";
			} else {
				t += '<script type="text/javascript">' 
					+ id + '._GLmol.create("'+id+'_appletdiv",true);' 
				if (Info.defaultModel)
					t += id + "._search(" + id + "._defaultModel);";
				if (this._readyScript)
					t += id + '._script(' + id + '._readyScript);' + "id" + "._showInfo(false)";
				t += '</script>';
			}
			t += Jmol._getWrapper(this, false);
			if (Info.addSelectionOptions)
				t += Jmol._getGrabberOptions(this, caption);
			if (Jmol._debugAlert && !Jmol._document)
				alert(t);
			this._code = Jmol._documentWrite(t);
		};
		
		proto._create2D = function(id, Info, caption) {
			Jmol._setObject(this, id, Info);
			var t = Jmol._getWrapper(this, true);
			//console.log(this);
			if (Jmol._document) {
				Jmol._documentWrite(t);
				this._createCanvas2d(id);
				t = "";
			} else {
				t += '<script type="text/javascript">' 
					+ id + '._createCanvas2d("'+id+'");' 
				if (Info.defaultModel)
					t += id + "._search(" + id + "._defaultModel);";
				if (this._readyScript)
					t += id + '._script(' + id + '._readyScript);' + "id" + "._showInfo(false)";
				t += '</script>';
			}
			t += Jmol._getWrapper(this, false);
			if (Info.addSelectionOptions)
				t += Jmol._getGrabberOptions(this, caption);
			if (Jmol._debugAlert && !Jmol._document)
				alert(t);
			this._code = Jmol._documentWrite(t);
		};
		
		proto._createCanvas2d = function(id) {
			var container = $('#' + id + "_appletdiv");
			var w = container.width();
			var h = container.height();
			var canvas = document.createElement( 'canvas' );
			canvas.applet = this;
			canvas.style.width = "100%";
			canvas.style.height = "100%";
			canvas.width = w;
			canvas.height = h; // don't understand why this is necessary
			canvas.id = id + "_canvas2d";
			container.append(canvas);
      Jmol._jsSetMouse(canvas);
		}
		
		proto._setupJS = function(id, Info) {
			if (!Info.console)
				Info.console = "JavaConsole";
			this.__Info = Info;
			window["j2s.lib"] = {
				base : Info.j2sPath + "/",
				alias : ".",
				console : Info.console
			};
			
			this._executionStack = [];
			this._executionStack.push([this.__createApplet, null])
			this._executionStack.push([this.__loadClass, "org.jmol.appletjs.Jmol"])
			if (!this._is2D) {
				this._executionStack.push([this.__addExportHook, null])
				this._executionStack.push([this.__loadClass, "org.jmol.exportjs.JSExporter"])
			}			 
			this._executionStack.push([this.__addURLHook, null])
			this._executionStack.push([this.__loadClass, "org.jmol.awtjs2d.JmolURLConnection"])
			this._executionStack.push([this.__loadClass, "org.jmol.awtjs.JmolURLConnection"])
			this._executionStack.push([this.__loadClazz])
			this._isSigned = true; // access all files via URL hook
			this._readyFunction = Info.readyFunction;
			this._ready = false; 
			this._applet = null;
			this._canScript = function(script) {return true;};
			this._savedOrientations = [];
			this._syncKeyword = "Select:";
			this.__nextExecution(this);
		};

		proto.__nextExecution = function(applet, trigger) {
		  if (applet._executionStack.length == 0)
		  	return;
		  if (!trigger) {
			  setTimeout(applet._id + ".__nextExecution(" + applet._id + ",true)",10)
		  	return;
		  }
		  var e = applet._executionStack.pop();
			if (e.length == 1)
				e[0](applet);
			else
				e[0](applet,e[1]);	
		};
				
		proto.__loadClazz = function(applet) {
			// problems with multiple applets?
		  if (!applet.__clazzLoaded) {
				LoadClazz();
				ClazzLoader.packageClasspath ("java", null, true);
				ClazzLoader.setPrimaryFolder (applet.__Info.j2sPath); // where org.jsmol.test.Test is to be found
				ClazzLoader.packageClasspath (applet.__Info.j2sPath); // where the other files are to be found
			}
			applet.__clazzLoaded = true;
			applet.__nextExecution(applet);
		};

		proto.__loadClass = function(applet, javaClass) {
		  ClazzLoader.loadClass(javaClass, function() {applet.__nextExecution(applet)});
		};
		proto.__addURLHook = function(applet) {
			org.jmol.awtjs.JmolURLConnection.prototype.doAjax = function() {
			 var data = Jmol._getFileData("" + this.url)
			 return org.jmol.util.StringXBuilder.newS(data);
			}
			org.jmol.awtjs2d.JmolURLConnection.prototype.doAjax = function() {
			 var data = Jmol._getFileData("" + this.url)
			 return org.jmol.util.StringXBuilder.newS(data);
			}
			applet.__nextExecution(applet);
		};
	
		proto.__addExportHook = function(applet) {

			// well, we don't want to assume there is only one applet, though....
			org.jmol.exportjs.JSExporter.prototype.jsInitExport = function(applet) {
				applet._GLmol.initializeJmolExport();
			}

			org.jmol.exportjs.JSExporter.prototype.jsSphere = function(applet, id, found, pt, o) {
				applet._GLmol.addJmolSphere(pt, o[0], o[1]);
			}
			org.jmol.exportjs.JSExporter.prototype.jsCylinder = function(applet, id, found, pt1, pt2, o) {
 				applet._GLmol.addJmolCylinder(pt1, pt2, o[0], o[1], o[2])
			}
			
			org.jmol.exportjs.JSExporter.prototype.jsTriangle = function(applet, color, pt1, pt2, pt3) {
 				applet._GLmol.addJmolTriangle(color, pt1, pt2, pt3)
			}
			
			org.jmol.exportjs.JSExporter.prototype.jsEndExport = function(applet) {
				applet._GLmol.finalizeJmolExport();
				applet._refresh();
			}
			
			/*
  private void jsSurface(Point3f[] vertices, Vector3f[] normals,
                         int[][] indices, int nVertices, int nPolygons,
                         int nFaces, BitSet bsPolygons, int faceVertexMax,
                         int color, int[] vertexColors, int[] polygonColors) {
    // JavaScript only    
  }
			*/
			
			org.jmol.exportjs.JSExporter.prototype.jsSurface = function(applet, vertices, normals, indices, 
					nVertices, nPolygons, nFaces, bsPolygons, faceVertexMax,
					color, vertexColors, polygonColors) {
			// notes: Color is only used if both vertexColors and polygonColors are null.
			//        Only one of vertexColors or polygonColors will NOT be null.
			//        Int facevertexMax is either 3 or 4; indices may have MORE than that number
			//        of vertex indices, because the last one may be a flag indicating which 
			//        edges to display when just showing mesh edges. When there are quadrilaterals,
			//        then nPolygons != nFaces, and you need to create both 3-sides and 4-sided faces
			//				based on the length of the individual indices[i] array.  
			
			// nFaces was determined as follows:
			
			//    boolean isAll = (bsPolygons == null);
			//    if (isAll) {
			//      for (int i = nPolygons; --i >= 0;)
			//        nFaces += (faceVertexMax == 4 && indices[i].length == 4 ? 2 : 1);    
			//    } else {
			//      for (int i = bsPolygons.nextSetBit(0); i >= 0; i = bsPolygons.nextSetBit(i + 1))
			//        nFaces += (faceVertexMax == 4 && indices[i].length == 4 ? 2 : 1);      
			
			
			    var params = {};
			    if (vertexColors != null) {
            params.vertexColors = THREE.VertexColors;
            var vc = new Array(vertexColors.length);
            for (var i = vertexColors.length; --i >= 0;)
              vc[i] = new THREE.Color(vertexColors[i]);
          } else if (polygonColors != null) {
            params.vertexColors = THREE.FaceColors;
          } else {
            params.color = color;
          }
			    var geo = new THREE.Geometry();
			    for (var i = 0; i < nVertices; i++) {
    				geo.vertices.push(new THREE.Vector3(vertices[i].x, vertices[i].y, vertices[i].z));
			    }
				for (var i = 0; i < nPolygons; i++) {
				  var h = indices[i][0], k = indices[i][1], l = indices[i][2];
          var m = indices[i][3];
				  var is3 = (faceVertexMax == 3 || indices[i].length == 3);
				  var f = (is3 ? new THREE.Face3(h, k, l) : new THREE.Face4(h, k, l, m));
				  // we can use the normals themselves, because they have .x .y .z
			    f.vertexNormals[0] = normals[h];
			    f.vertexNormals[1] = normals[k];
			    f.vertexNormals[2] = normals[l];
			    if (is3) {
  			    if (vertexColors != null) {
              f.vertexColors = [vc[h], vc[k], vc[l]];
            }
			    } else {
				    f.vertexNormals[3] = normals[m];
			    }

          if (polygonColors != null) {
            f.color = new THREE.Color(polygonColors[i]);
          }

				  geo.faces.push(f);
				}

			    var obj = new THREE.Mesh(geo, new THREE.MeshLambertMaterial(params));
			    obj.doubleSided = true; // generally?
//			    obj.material.wireframe = true;
			    applet._GLmol.modelGroup.add(obj);

//			    console.log(obj);
			}
			applet.__nextExecution(applet);
		};

		proto.equals = function(a) { return this == a };
		proto.clone = function() { return this };
		proto.hashCode = function() { return parseInt(this._syncId) };  
			
		proto.__createApplet = function(applet) {
		
			var viewerOptions =  new java.util.Hashtable ();
			var availableValues = "'messagecallback','pickcallback','animframecallback','appletreadycallback','atommovedcallback',\
									'echocallback','evalcallback','hovercallback','language','loadstru','measurecallback',\
									'minimizationcallback','resizecallback','scriptcallback','statusform','statustext','statustextarea',\
									'synccallback'";
			for (var i in applet.__Info)
				if(availableValues.indexOf("'" + i.toLowerCase() + "'") >= 0)
					viewerOptions.put(i, applet.__Info[i]);
			viewerOptions.put("appletReadyCallback","Jmol._readyCallback");
			viewerOptions.put("applet", true);
			viewerOptions.put("name", applet._id + "_object");
			viewerOptions.put("syncId", applet._syncId);
			viewerOptions.put("script", "if(!_is2D){set multipleBondSpacing 0.35};" + (!applet.__Info.color ? "" :
			  "background " + 
					(applet.__Info.color.indexOf("#") == 0 ? "[0x" + applet.__Info.color.substring(1) + "]" : applet.__Info.color)))

			viewerOptions.put("signedApplet", "true");
			viewerOptions.put("platform", applet._platform);
			if (applet._is2D)
				viewerOptions.put("display",applet._id + "_canvas2d");
				
			//viewerOptions.put("repaintManager", "org.jmol.render");
			viewerOptions.put("documentBase", document.location.href);
			var base = document.location.href.split("?")[0].split("#")[0].split("/")
			base[base.length - 1] = window["j2s.lib"].base
			viewerOptions.put ("codeBase", base.join("/"));
			Jmol._registerApplet(applet._id, applet)
			applet._applet = new org.jmol.appletjs.Jmol(viewerOptions);
			if (!applet._is2D)
				applet._GLmol.applet = applet;
			applet._jsSetScreenDimensions();
			
			applet.__nextExecution(applet);
		};
		
		proto._jsSetScreenDimensions = function() {
				if (!this._applet)return
				// strangely, if CTRL+/CTRL- are used repeatedly, then the
        // applet div can be not the same size as the canvas if there
        // is a border in place.
				var d = Jmol._getElement(this, (this._is2D ? "canvas2d" : "canvas"));
				this._applet.viewer.setScreenDimension(
				d.width, d.height);
//				Math.floor($(Jmol._getElement(this, "appletdiv")).height()));

		};

		proto._loadModel = function(mol, params) {
			var script = 'load DATA "model"\n' + mol + '\nEND "model" ' + params;
			this._script(script);
			this._showInfo(false);
		// temporary only:
		//	var glcanvas = this._GLmol;
		//	glcanvas.emptyMessage="Retrieving data...";
		//	glcanvas._jmolFileType = Jmol._getFileType(this._fileName);
 		//	Jmol._glProcessFileData(glcanvas,  mol, "");
		
		};
	
		
		proto._show = function(tf) {
			var w = (!tf ? "2px" : "100%");
			var h = (!tf ? "2px" : "100%");
			//var w = (!tf ? "2px" : this._containerWidth.indexOf("px") >= 0 ? this._containerWidth : "100%");
			//var h = (!tf ? "2px" : this._containerHeight.indexOf("px") >= 0 ? this._containerHeight : "100%");
			var o = Jmol._getElement(this, "object");
			if (o && o.style) {
				o.style.width = w; 
				o.style.height = h;
			}
			this._jsSetScreenDimensions();
		};
		

		proto._canScript = function(script) {return true};
		
    /**
     * @j2sNative
     * if ((mode == 2 || mode == 7) && typeof Jmol != "undefined") {
     *   this.transformManager.finalizeTransformParameters();
     *   Jmol.refresh(this.htmlName);
     * }
     */
		proto._refresh = function() {
			
	// OK, here is where we need to implement the new camera/rotation positions. Just guessing here
	// pixelsPerAngstrom can be used to calculate the new camera position.
	// modelRadius is half the distance across the screen. 
	// 100% implies that camera Z position is 3.5 * modelRadius, with a field of view of 16.24 degrees (2 * atan(1/7))
	// Since Jmol changes the size of the model, not the camera position, I should change the way this is all reported.
	// First I need to see a model!
	
			if (!this._applet || !this._applet.viewer)
				return
			var gl = this._GLmol;
			var rg = gl.rotationGroup;
			if (!rg)
				return
			var mg = gl.modelGroup;
			var tm = this._applet.viewer.transformManager;
			var rQ = tm.getRotationQuaternion();
			rg.quaternion = new THREE.Quaternion(-rQ.q3, -rQ.q0, rQ.q1, rQ.q2);
			// cameraDistance is in units of screenPixelCount; distance is to front of scene, not to the center.
			// scaled linearly by zoom
			var sppa = tm.scalePixelsPerAngstrom;
			rg.position.z = gl.CAMERA_Z 
				+ (tm.cameraDistance+tm.screenPixelCount*0.5)/sppa;
			//model "position" is moved such that {0 0 0} is the fixedRotationCenter 
			mg.position.x = -tm.fixedRotationCenter.x; 
			mg.position.y = -tm.fixedRotationCenter.y;
			mg.position.z = -tm.fixedRotationCenter.z; 
			//there is also the fixedTranslation to worry about (from CTRL-ALT-drag)
			if (tm.perspectiveDepth) {
				gl.camera = gl.perspectiveCamera;
				gl.camera.fov = gl.fov;
			} else {
				gl.camera = gl.orthoscopicCamera;
				gl.camera.right = tm.width * 0.5 / sppa;
				gl.camera.left = -gl.camera.right;
				gl.camera.top = tm.height * 0.5 / sppa;
				gl.camera.bottom = -gl.camera.top;
			}
			gl.camera.updateProjectionMatrix();
		  var me = this;
		  requestAnimationFrame(function() {me._GLmol.show();});
		}
		
		proto._loadFile = function(fileName, params){
		// temporary only
			this._showInfo(false);
			params || (params = "");
			this._thisJmolModel = "" + Math.random();
			this._fileName = fileName;
			this._script("zap;set echo middle center;echo Retrieving data...");
			if (false && this._isSigned) {
				this._script("load \"" + fileName + "\"" + params);
				return;
			}
			var self = this;
			Jmol._loadFileData(this, fileName, function(data){self._loadModel(data, params)});
		};
    return proto;
	};
	
  Jmol._refresh = function(applet, mode, strWhy, data) {
  	// just the orientation/scaling
    if (applet && applet._refresh) {
   		applet._refresh(mode, strWhy, data)
    }
	}

  Jmol._repaint = function(applet, asNewThread) {
    // requires rebuilding via methods of classes in org.jmol.exportjs, which will come back from updateJS
	  // could check for resize here
		//System.out.println("Jmol._repaint " + applet._id + " " + asNewThread)
  	if (asNewThread) {
      setTimeout(function(){ applet._applet.viewer.updateJS(0,0)});
  	} else {
  		applet._applet.viewer.updateJS(0,0);
  	}
	}

GLmol.prototype.initializeJmolExport = function() {
	this._view = this.getView();
	this.initializeScene();
	this.tpt1 = new TV3();
	this.tpt2 = new TV3();
	this._m = new THREE.Matrix4();
	this._geoTriangles = null;
	
	if (!this.sphereGeometry) {
		this.sphereGeometry = new THREE.SphereGeometry(1, this.sphereQuality, this.sphereQuality); // r, seg, ring
		this.sphereGeometry0 = new THREE.SphereGeometry(1, this.sphereQuality / 4, this.sphereQuality / 4); // r, seg, ring
		this.cylinderGeometry = {};
		
		this.cylinderGeometry['c'] = new THREE.CylinderGeometry(1, 1, 1, this.cylinderQuality, 1, true);
		this.cylinderGeometry['c'].faceUvs = [];
		this.cylinderGeometry['c'].faceVertexUvs = [];
		this.cylinderGeometry['c0'] = new THREE.CylinderGeometry(1, 1, 1, this.cylinderQuality/4, 1, true);
		this.cylinderGeometry['c0'].faceUvs = [];
		this.cylinderGeometry['c0'].faceVertexUvs = [];
		this.cylinderGeometry['c'] = new THREE.CylinderGeometry(1, 1, 1, this.cylinderQuality, 1, true);
		this.cylinderGeometry['c'].faceUvs = [];
		this.cylinderGeometry['c'].faceVertexUvs = [];
		this.cylinderGeometry['c0'] = new THREE.CylinderGeometry(1, 1, 1, this.cylinderQuality/4, 1, true);
		this.cylinderGeometry['c0'].faceUvs = [];
		this.cylinderGeometry['c0'].faceVertexUvs = [];

		this.cylinderGeometry['n'] = new THREE.CylinderGeometry(1, 1, 1, this.cylinderQuality, 1, false);
		this.cylinderGeometry['n'].faceUvs = [];
		this.cylinderGeometry['n'].faceVertexUvs = [];
		this.cylinderGeometry['n0'] = new THREE.CylinderGeometry(1, 1, 1, this.cylinderQuality/4, 1, false);
		this.cylinderGeometry['n0'].faceUvs = [];
		this.cylinderGeometry['n0'].faceVertexUvs = [];
		this.cylinderGeometry['n'] = new THREE.CylinderGeometry(1, 1, 1, this.cylinderQuality, 1, false);
		this.cylinderGeometry['n'].faceUvs = [];
		this.cylinderGeometry['n'].faceVertexUvs = [];
		this.cylinderGeometry['n0'] = new THREE.CylinderGeometry(1, 1, 1, this.cylinderQuality/4, 1, false);
		this.cylinderGeometry['n0'].faceUvs = [];
		this.cylinderGeometry['n0'].faceVertexUvs = [];

	}
};

GLmol.prototype.drawSphere = function(group, pt, color, radius) {
  var isRough = (radius < 0.05); // typical mesh dot
	var geom = (isRough ? this.sphereGeometry0 : this.sphereGeometry);
	var sphere = new THREE.Mesh(geom, GLmol.getMat(this, color));
	group.add(sphere);
	sphere.scale.x = sphere.scale.y = sphere.scale.z = radius;
	sphere.position.x = pt.x;
	sphere.position.y = pt.y;
	sphere.position.z = pt.z;
}
		
GLmol.prototype.addJmolSphere = function(pt, color, radius) {
	this.drawSphere(this.modelGroup, pt, color, radius);
}

GLmol.setPt = function(p,q) {p.x = q.x;p.y=q.y;p.z=q.z;};

GLmol.prototype.addJmolCylinder = function(pt1, pt2, color, length, radius) {
	GLmol.setPt(this.tpt1,pt1);
	GLmol.setPt(this.tpt2,pt2);
	this.drawCylinder(this.modelGroup, this.tpt1, this.tpt2, 0 + radius, color, true);
};
			
GLmol.prototype.addJmolTriangle = function(color, pt1, pt2, pt3) {
	var geo = this._geoTriangles;
	if (!geo)
		geo = this._geoTriangles = new THREE.Geometry();
	var n = geo.vertices.length;
	geo.vertices.push(new THREE.Vector3(pt1.x, pt1.y, pt1.z));	
	geo.vertices.push(new THREE.Vector3(pt2.x, pt2.y, pt2.z));	
	geo.vertices.push(new THREE.Vector3(pt3.x, pt3.y, pt3.z));	
	var f = new THREE.Face3(n, n+1, n+2, null, new THREE.Color(color));
	geo.faces.push(f);
}

GLmol.prototype.finalizeJmolExport = function() {	

	if (this._geoTriangles) {
		var geo = this._geoTriangles;
		geo.computeFaceNormals();
		var params = {vertexColors: THREE.FaceColors};
    var obj = new THREE.Mesh(geo, new THREE.MeshLambertMaterial(params));
    obj.doubleSided = true; // generally?
//			    obj.material.wireframe = true;
    this.modelGroup.add(obj);

	}
	this.setView(this._view);
};

GLmol.prototype.drawCylinder = function(group, from, to, radius, color, cap) {
	if (!from || !to) return;

	midpoint = new TV3().add(from, to).multiplyScalar(0.5);
	
	var isRough = (radius < 0.05);
	var geom = this.cylinderGeometry[(cap ? "c" : "n") + (isRough ? "0" : "")];
	var cylinder = new THREE.Mesh(geom, GLmol.getMat(this, color));
	cylinder.position = midpoint;
	cylinder.lookAt(from);
	cylinder.updateMatrix();
	cylinder.matrixAutoUpdate = false;
	cylinder.matrix.multiplySelf(this._m.makeScale(radius, radius, from.distanceTo(to)).rotateX(Math.PI / 2));
	group.add(cylinder);
};

Jmol._getHiddenCanvas = function(applet, id, width, height) {
	id = applet._id + "_" + id;
	var d = document.getElementById(id);
	if (!d)
    d = document.createElement( 'canvas' );
    // for some reason both these need to be set, or maybe just d.width?
	d.width = d.style.width = width;
	d.height = d.style.height = height;
	d.style.display = "none";
	if (d.id != id) {
		d.id = id;
  	$("body").append(d);
  }
  return d;
}

})(Jmol);

